from optparse import OptionParser
from utilities.align_parsers import psl
import re

class BubbleMapper:
    
    def __init__(self, align_file):
        self.align_file = align_file
        self.bubble2contig = {}
        
    @classmethod
    def split_name(cls, name):
        bubble_name = re.compile(r'(?P<num>\S+)(?P<branch>[A-Z])')
        m = bubble_name.search(name)
        if m:
            return m.group('num'), m.group('branch')
        else:
            return None, None
        

    def map_to_contigs(self):
        aligns = psl.parse(self.align_file, noline=True)

        mapping = {}
        for align in aligns:
            num, branch = BubbleMapper.split_name(align.query)
        
            if not mapping.has_key(num):
                mapping[num] = {}
            
            if not mapping[num].has_key(branch):
                mapping[num][branch] = {}
            
            mapping[num][branch][align.target] = [align.tstart, align.match]
        
        for num in sorted(mapping.keys()):        
            branches = sorted(mapping[num].keys())
        
            for i in range(len(branches)):
                targets = mapping[num][branches[i]]
            
                for target in targets:
                    matches = {mapping[num][branches[i]][target][1]:True}
                    agreed = 1
                    #print num, i, branches[i], target, agreed, matches
                    for j in range(len(branches)):
                        #print num,'j', j
                        if j != i:
                            if mapping[num][branches[j]].has_key(target):
                            #if mapping[num][branches[j]].has_key(target) and mapping[num][branches[j]][target][0] == mapping[num][branches[i]][target][0]:
                                agreed += 1
                                matches[mapping[num][branches[j]][target][1]] = True
                                #print num, j, branches[j], target, agreed, matches
                                continue
                    #print num, branches[i], target, agreed, matches
                    
                    if agreed == len(branches) and len(matches.keys()) > 1:
                        for branch in branches:
                            self.bubble2contig[num + branch] = target
                        break
                    
                        
    def is_bubble_mapped_to_contig(self, name):
        if self.bubble2contig.has_key(name):
            return self.bubble2contig[name]
        else:
            return False
        
if __name__ == '__main__':
    usage = "Usage: %prog align-file"
    parser = OptionParser(usage=usage)

    (options, args) = parser.parse_args()
    bm = BubbleMapper(args[0])
    bm.map_to_contigs()
    print bm.is_bubble_mapped_to_contig('7358B')