import sys, re, os, ConfigParser
from optparse import OptionParser
from utilities.bam import BAM
from utilities.track import Track
from utilities.intspan import cardinality
import subprocess
from event import Event
from transcript import Transcript
from utilities.tools import set_attrs, bamfile_contig_name
from annotations import ensembl, knownGene, refGene, aceview, ensg

PACKAGE_DIR = "/".join(os.path.abspath(sys.argv[0]).split("/")[:-2])

def reform_events(events_file, track, txts_dict):
    events = []

    #get alignments
    trk = Track(track)
    aligns = trk.parse_psl(minimum=True)
    aligns_dict = {}
    for a in aligns:
        aligns_dict[a.query] = a
    
    for line in open(events_file, 'r'):
        cols = line.rstrip("\n").split("\t")

        #skip header
        if cols[0] == 'id':
            continue

        data = {}

        headers = Event.headers[:]
        headers.insert(0, 'id')

        for i in range(len(headers)):
            data[headers[i]] = cols[i]

        data['filter'] = None

        if data['align_blocks'] != 'na':
            data['align_blocks'] = [int(n) for n in data['align_blocks'].split(',')]
        else:
            data['align_blocks'] = None

        if data['type'] != 'novel_transcript':
            if data['exons']:
                data['exons'] = [int(n) for n in data['exons'].split(',')]
            else:
                data['exons'] = None


        if aligns_dict.has_key(data['contig']):
            align = aligns_dict[data['contig']]
        else:
            align = None

        #coords = re.split('[:-]', data['coord'])
        data['chrom'] = re.split('[:-]', data['coord'])[0]
        data['align_coords'] = []
        if data['type'] in ('AS3', 'AS5'):
            data['align_coords'] = align.blocks[data['align_blocks'][0]-1]
        else:
            for i in data['align_blocks']:
                data['align_coords'].append(align.blocks[i-1])

        if type == 'novel_transcript':
            txt = None
        elif txts_dict.has_key(data['transcript']):
            txt = txts_dict[data['transcript']]

            data['exon_coords'] = []
            for i in data['exons']:
                data['exon_coords'].append(txt.exons[i-1])
        else:
            txt = None

        event = Event(data['type'])
        set_attrs(event, data)
        
        event.txt = txt
        event.align = align

        events.append(event)

    return events

def set_read_support(event, genome=False, min_spanning_reads=2, max_coverage_diff=100):
    read_support = True
    
    if event.type in ('novel_exon', 'novel_utr', 'AS53'):
        if not event.coverage:
            read_support = False
        else:
            if type(event.coverage).__name__ == 'list':
                contig_coverage = min(event.coverage)
            elif event.coverage == 'na':
                contig_coverage = event.coverage
            else:
                contig_coverage = int(event.coverage)

            if contig_coverage < min_spanning_reads:
                read_support = False
            

    elif event.type == 'AS5' or event.type == 'AS3':
        if event.spanning_reads < min_spanning_reads:
            read_support = False

    elif event.type == 'retained_intron':
        if not event.coverage:
            read_support = False
        else:
            if type(event.coverage).__name__ == 'list':
                contig_coverage = min(event.coverage)
            else:
                contig_coverage = int(event.coverage)

            if contig_coverage < min_spanning_reads:
                read_support = False
        
                
    elif event.type == 'skipped_exon':
        #don't check genome coverage because there may be other alleles that don't have skipped exon
        if int(event.spanning_reads) < min_spanning_reads:
            read_support = False

    elif event.type == 'novel_intron':
        if int(event.spanning_reads) < min_spanning_reads:
            read_support = False
        
    elif event.type == 'novel_transcript':
        valid_links = [l for l in event.coverage if int(l) > min_spanning_reads]

        if len(valid_links) < len(event.coverage):
            read_support = False

    event.read_support = read_support


def find_support_contig(event, contig_bam, lib, outdir=None):
    cname = bamfile_contig_name(event.contig, lib)

    align = event.align
    target = align.target

    blocks = event.align_blocks
    exons = event.exons

    event.coverage = event.spanning_reads = None
    
    if event.type in ('skipped_exon', 'novel_intron'):
        pos = [align.query_blocks[blocks[0]-1][1], align.query_blocks[blocks[1]-1][0]]
        pos.sort(key = int)
        event.contig_coord = '-'.join([str(pos[0]), str(pos[1])])
        spanning_reads = read_support(contig_bam, cname, pos, junction=True, seq=True)

        event.spanning_reads = len(spanning_reads)
        #event.coverage = event.spanning_reads

        if outdir:
            output_spanning_reads(event, outdir, spanning_reads, pos)

    elif event.type in ('AS53', 'novel_exon', 'novel_utr'):
        event.spanning_reads = []
        event.coverage = []
        all_spanning_reads = []
        
        #for block in blocks:
            #unique_reads = {}
            #if block - 2 >= 0:
                #pos = [align.query_blocks[block-2][1], align.query_blocks[block-1][0]]
                #spanning_reads = read_support(contig_bam, cname, pos, junction=True, seq=True)
                #for read in spanning_reads:
                    #unique_reads[read.qname + read.seq] = read

            #if block <= len(align.query_blocks)-1:
                #pos = [align.query_blocks[block-1][1], align.query_blocks[block][0]]
                #spanning_reads = read_support(contig_bam, cname, pos, junction=True, seq=True)
                #for read in spanning_reads:
                    #unique_reads[read.qname + read.seq] = read
                
            #event.spanning_reads.append(len(unique_reads.keys()))
            #all_spanning_reads.extend(unique_reads.values())
            
            ##coverage
            #event.coverage.append(read_support(contig_bam, cname, pos, block=True))
            
        
        block = blocks[0]
        num_spanning_reads = [0,0]
        contig_coords = ['na', 'na']
        unique_reads = {}
        if block - 2 >= 0:
            pos = [align.query_blocks[block-2][1], align.query_blocks[block-1][0]]
            spanning_reads = read_support(contig_bam, cname, pos, junction=True, seq=True)
            num_spanning_reads[0] = len(spanning_reads)
            for read in spanning_reads:
                unique_reads[read.qname + read.seq] = read
            contig_coords[0] = str(pos[0]) + '-' + str(pos[1])
        

        if block <= len(align.query_blocks)-1:
            pos = [align.query_blocks[block-1][1], align.query_blocks[block][0]]
            spanning_reads = read_support(contig_bam, cname, pos, junction=True, seq=True)
            num_spanning_reads[1] = len(spanning_reads)
            for read in spanning_reads:
                unique_reads[read.qname + read.seq] = read
            contig_coords[1] = str(pos[0]) + '-' + str(pos[1])
                
        event.spanning_reads = str(num_spanning_reads[0]) + ',' + str(num_spanning_reads[1])
        event.coverage = read_support(contig_bam, cname, align.query_blocks[block-1], block=True, seq=False)
        event.contig_coord = ','.join(contig_coords)
                
        if outdir:
            output_spanning_reads(event, outdir, unique_reads.values(), pos)       

    elif event.type == 'retained_intron':
        block = blocks[0]
        offset1 = event.txt.exons[exons[0]-1][1] - align.blocks[block-1][0]
        offset2 = align.blocks[block-1][1] - event.txt.exons[exons[1]-1][0]
        if align.query_blocks[block-1][0] < align.query_blocks[block-1][1]:
            pos = [align.query_blocks[block-1][0]+offset1+1, align.query_blocks[block-1][1]-offset2-1]
        else:
            pos = [align.query_blocks[block-1][0]-offset1-1, align.query_blocks[block-1][1]+offset2+1]
        pos.sort(key = int)
        event.contig_coord = '-'.join([str(pos[0]), str(pos[1])])

        spanning_reads = read_support(contig_bam, cname, pos, block=True, seq=True)

        #event.spanning_reads = len(spanning_reads)
        event.coverage = len(spanning_reads)

        if outdir:
            output_spanning_reads(event, outdir, spanning_reads, pos)

    elif event.type == 'AS5' or event.type == 'AS3':
        #event.expanded = event.shrunk = False
        #left
        if (event.type == 'AS5' and event.txt.strand == '+') or (event.type == 'AS3' and event.txt.strand == '-'):
            if blocks[0] > 1:
                pos = [align.query_blocks[blocks[0]-2][1], align.query_blocks[blocks[0]-1][0]]
            else:
                pos = [align.query_blocks[blocks[0]-1][0], align.query_blocks[blocks[0]-1][0]]
        #right
        else:
            if blocks[0] < len(align.query_blocks):
                pos = [align.query_blocks[blocks[0]-1][1], align.query_blocks[blocks[0]][0]]
            else:
                pos = [align.query_blocks[blocks[0]-1][1], align.query_blocks[blocks[0]-1][1]]

        pos.sort(key = int)
        event.contig_coord = '-'.join([str(pos[0]), str(pos[1])])
        spanning_reads = read_support(contig_bam, cname, pos, junction=True, seq=True)
        event.spanning_reads = len(spanning_reads)
        #event.coverage = event.spanning_reads

        if outdir:
            output_spanning_reads(event, outdir, spanning_reads, pos)
    
        
    elif event.type == "novel_transcript":
        spanning_reads = []
        coverage = []
        contig_coords = []
        for i in range(len(align.query_blocks)):
            j = i + 1

            reads = read_support(contig_bam, cname, align.query_blocks[i], block=True, seq=True)
            coverage.append(str(len(reads)))
            
            contig_coords.append(str(align.query_blocks[i][0]) + '-' + str(align.query_blocks[i][1]))
            
            if j < len(align.query_blocks):
                pos = [align.query_blocks[i][1], align.query_blocks[j][0]]
                reads = read_support(contig_bam, cname, pos, junction=True, seq=True)
                spanning_reads.append(str(len(reads)))
                
        event.spanning_reads = ','.join(spanning_reads)
        event.coverage = ','.join(coverage)
        event.contig_coord = ','.join(contig_coords)

def output_spanning_reads(event, outdir, spanning_reads, pos):
    if os.path.isdir(outdir):
        out = open(outdir + "/" + '::'.join((event.contig, event.type, event.coordinate())) + '.fa', 'w')
        for read in spanning_reads:
            out.write('>' + '-'.join((read.qname, str(read.pos))) + "\n")
            out.write(read.seq + "\n")
        out.close()

def screen(event):
    if event.read_support:
        event.filter = 'passed'
    else:
        event.filter = 'failed'
                    
    if event.filter == 'passed' and event.type in ('AS5', 'AS3', 'novel_exon', 'novel_utr', 'novel_intron', 'AS53', 'novel_transcript') and event.splice and '?' in event.splice:
        event.filter = 'failed'

    if event.filter == 'passed' and event.type == 'retained_intron' and not 'True' in event.multi_3:
        event.filter = 'failed'

    elif event.type == 'novel_transcript':
        if event.splice and not '?' in event.splice:
            event.filter = 'passed'
        else:
            event.filter = 'failed'
        

def read_support(contig_bam, contig, pos=None, junction=False, block=None, min_span=4, seq=False):
    reads = []
    if pos is None:
        return reads
    
    pos.sort(lambda x,y: x-y)
    if junction:
        pos_span = [pos[0]-min_span+1, pos[1]+min_span-1]
        reads = contig_bam.confirm(contig, pos_span, feature='breakpoint', seq=seq)
    elif block:
        reads = contig_bam.coverage_simple(contig, pos, total=True, seq=seq)
        
    return reads

def get_txts(genome):
    config_file = PACKAGE_DIR + "/configs/model_matcher.cfg"
    
    config = ConfigParser.ConfigParser()
    config.read(config_file)

    annot_files = {}
    if genome in config.sections():
        for name in config.options(genome):
            if name != 'order' and name != 'splice' and name != 'path':
                annot_files[name] = config.get(genome, name)

    txts_dict = {}
    for model, file_name in annot_files.iteritems():
        path = PACKAGE_DIR + "/annotations/" + genome + "/" + file_name

        txts_dict = {}

        if os.path.exists(path):
            txts = {
                'e': ensembl.parse,
                'r': refGene.parse,
                'k': knownGene.parse,
                'a': aceview.parse,
                'x': ensg.parse
                }[model](path)

            for txt in txts:
                txts_dict[txt.name] = txt

    return txts_dict
    
def get_txt(txts_dict, model, name):
    if txts_dict.has_key(model) and txts_dict[model].has_key(name):
        return txts_dict[model][name]

    return None

def output_events(events, outfile):
    out = open(outfile, 'w')

    headers = Event.headers[:]
    headers.insert(0, 'id')
    headers.extend(Event.headers_support)
    out.write('\t'.join(headers) + "\n")

    for e in events:
        out.write(e.id + "\t" + e.output() + "\n")

    out.close()
    

if __name__ == '__main__':
    usage = "Usage: %proj events_file track contigs-bamfile library genome output"

    parser = OptionParser(usage=usage)
    parser.add_option("-r", "--out_reads", dest="out_reads", help="output supporting reads dir")
    parser.add_option("-g", "--genome_bam", dest="genome_bam", help="genomic bam file")
    parser.add_option("-R", "--min_reads", dest="min_spanning_reads", help="mininum number of spanning reads. Default:2", type='int', default=2)
    parser.add_option("-X", "--max_cov_diff", dest="max_coverage_diff", help="maximum coverage difference. Default:100", type='int', default=100)

    (options, args) = parser.parse_args()

    #reform events
    txts_dict = get_txts(args[4])    
    events = reform_events(args[0], args[1], txts_dict)
    sys.stderr.write("finished capturing events\n")

    #get bam files
    contig_bam = BAM(args[2], min_mapq=0)

    genome_bam = None
    if options.genome_bam:
        genome_bam = BAM(options.genome_bam, min_mapq=1)
    
    lib = args[3]

    #screen events
    if contig_bam:
        for event in events:
            find_support_contig(event, contig_bam, lib, outdir=options.out_reads)

    use_genome = False
    if genome_bam:
        use_genome = True
        for event in events:
            find_support_genome(event, genome_bam)

    for event in events:
        set_read_support(event, use_genome, min_spanning_reads=options.min_spanning_reads, max_coverage_diff=options.max_coverage_diff)
        screen(event)
                
    #output events
    output_events(events, args[-1])

