from utilities.bam import BAM
import re, os, sys
from utilities import tools

class SNV:
    
    def __init__(self, method=None, snv_type=None, ref=None, ref_start=None, ref_end=None, ref_seq=None, query_strand=None, var=None, var_start=None, var_end=None, var_seq=None, var_len=None):
        self.method = method
        self.snv_type = snv_type
        self.var = var
        self.var_start = var_start
        self.var_end = var_end
        self.var_seq = var_seq
        self.ref = ref
        self.ref_start = ref_start
        self.ref_end = ref_end
        self.ref_seq = ref_seq
        self.var_len = var_len
	
        if (snv_type == 'ins' or snv_type == 'inv') and var_seq:
            self.snv_len = len(var_seq)
        elif ref_seq:
            self.snv_len = len(ref_seq)

        self.from_end = None
        self.nreads_contig = 'na'
        self.nreads_genome = 'na'
	#from contig alignment
	self.nreads_event = 'na'
        self.gene = 'na'
	self.query_strand = query_strand
	
	self.expansion = 0

        if self.var_start and self.var_end:
            self.confirm_contig_region = [int(self.var_start), int(self.var_end)]
	else:
	    self.confirm_contig_region = []
	    
	#filtering
	self.enough_coverage = False
	self.exon = False
	self.nonsynon = False
	self.too_close_to_end = False
	self.at_least_1_read_opposite = False
	
	#repeats
	self.within_simple_repeats = '-'
	self.repeatmasker = '-'
	self.within_segdup = '-'
	
	#for filtering snvs from bad contig region
	self.artefact = False
	
	self.dbsnp = '-'
	
    def gff(self, type):
        "output gff format"

	if re.match('^(chr|scaffold)', self.ref, re.IGNORECASE):
            chrom = self.ref
        else:
            chrom = 'chr' + self.ref

        var_shortened = self.var.split(" ")[0]

        if self.snv_type.upper() == "SNV":
            label_last = self.ref_seq.upper() + "->" + self.var_seq.upper()
        else:
            label_last = self.snv_type.upper()
        label = ":".join([var_shortened, chrom, str(self.ref_start), label_last])

        out = ""

        out = "\t".join([chrom, self.method, type, str(self.ref_start), str(self.ref_end), ".", '+', ".", label])
        return out + "\n"


    def details(self):
        return "%s %s:%d-%d %s:%d-%d %d %s->%s %s %s" % (self.snv_type, self.ref, self.ref_start, self.ref_end, self.var, self.var_start, self.var_end, self.snv_len, self.ref_seq, self.var_seq, self.query_strand, self.expansion)

    def tab(self, debug=False):
        "tabular summary of snv"
        
	chrom = self.ref

        var_shortened = self.var.split(" ")[0]

        ref = alt = 'na'
        if self.snv_type == "snv" or self.snv_type == 'inv':
            ref = self.ref_seq
            alt = self.var_seq
        elif self.snv_type == "del":
	    if len(self.ref_seq) > 10:
		ref = self.ref_seq[:5] + '...' + self.ref_seq[-5:]
	    else:
		ref = self.ref_seq
        elif self.snv_type == "ins":
            alt = self.var_seq
	    
	if not self.gene:
	    self.gene = 'na'
	    
	if not self.at_least_1_read_opposite:
	    at_least_1_read_opposite = 'false'
	else:
	    at_least_1_read_opposite = 'true'
	    	
	fields = [self.snv_type, chrom, str(self.ref_start), str(self.ref_end), var_shortened, str(self.var_len), str(self.var_start), str(self.var_end), str(self.snv_len), ref, alt, str(self.nreads_event), str(self.nreads_contig), str(self.nreads_genome), self.gene, str(self.expansion), self.query_strand, str(self.from_end), "%s-%s" % (self.confirm_contig_region[0], self.confirm_contig_region[1]), self.within_simple_repeats, self.repeatmasker, self.within_segdup, at_least_1_read_opposite, self.dbsnp]
	
	if debug:
	    fields.append("enough_reads:" + str(self.enough_coverage))
	    fields.append("too_close_to_end:" + str(self.too_close_to_end))
	    fields.append("at_least_1_read_opposite:" + str(self.at_least_1_read_opposite))
	    fields.append("exon:" + str(self.exon))
	    fields.append("nonsynon:" + str(self.nonsynon))
	    
	return "\t".join(fields) + "\n"


    def coord(self):
        return "%s:%s-%s" % (self.ref, self.ref_start, self.ref_end)

    def genome_read_support(self, bam, refseq, from_end=8):
	if re.match('^(chr|scaffold)', self.ref, re.IGNORECASE):
            chrom = self.ref
        else:
            chrom = 'chr' + self.ref
	    
	if self.snv_type == 'ins' or self.snv_type == 'del':       
	    if self.snv_type == 'del':
		self.nreads_genome = bam.confirm(chrom, [int(self.ref_start),int(self.ref_end)], feature=self.snv_type, refseq=refseq)
	    elif self.snv_type == 'ins':
		self.nreads_genome = bam.confirm(chrom, [int(self.ref_start)], feature=self.snv_type, allele=self.var_seq, refseq=refseq)
		
        elif self.snv_type == 'snv':
            self.nreads_genome = bam.confirm(chrom, [int(self.ref_start)], feature=self.snv_type, allele=self.var_seq)
	    
	elif self.snv_type == 'inv':
	    self.nreads_genome = bam.confirm_perfect(chrom, [int(self.ref_start),int(self.ref_end)], self.var_seq, '+', min_from_end=from_end)
	    
	return self.nreads_genome

    def contig_read_support(self, bam, lib=None, from_end=8, get_reads=False):
        ctg = self.var.split(" ")[0]	    
	ctg_name = tools.bamfile_contig_name(ctg, lib)
	
        reads = []
        if self.snv_type == 'ins':
	    if int(self.var_end)-int(self.var_start)+1 != int(self.snv_len):
                self.nreads_contig = 0
            else:
                if get_reads:
		    reads = bam.confirm_perfect(ctg_name, self.confirm_contig_region, self.var_seq, self.query_strand, min_from_end=from_end, expansion=self.expansion, seq=True)
                    self.nreads_contig = len(reads)
                else:
		    self.nreads_contig = bam.confirm_perfect(ctg_name, self.confirm_contig_region, self.var_seq, self.query_strand, min_from_end=from_end, expansion=self.expansion)
                    
        elif self.snv_type == 'snv' or self.snv_type == 'inv':
            if int(self.var_end)-int(self.var_start)+1 != int(self.snv_len):
                self.nreads_contig = 0
            else:	
		if get_reads:
		    reads = bam.confirm_perfect(ctg_name, self.confirm_contig_region, self.var_seq, self.query_strand, min_from_end=from_end, seq=True)
		    self.nreads_contig = len(reads)
		else:
		    self.nreads_contig = bam.confirm_perfect(ctg_name, self.confirm_contig_region, self.var_seq, self.query_strand, min_from_end=from_end)
		
        elif self.snv_type == 'del':
            if get_reads:
                reads = bam.confirm(ctg_name, [int(self.confirm_contig_region[0])-from_end+1,int(self.confirm_contig_region[1])+from_end-1], feature='breakpoint', seq=True)
                self.nreads_contig = len(reads)
            else:
                self.nreads_contig = bam.confirm(ctg_name, [int(self.confirm_contig_region[0])-from_end+1, int(self.confirm_contig_region[1])+from_end-1], feature='breakpoint')

        if get_reads:
	    return reads

    def upshift(self, refseq):       
        if self.snv_type == 'ins' or self.snv_type == 'del':
            if self.snv_type == 'ins':
                if tools.is_homopolymer(self.var_seq):
                    #print "is homopolymer", self.var_seq
                    size = 1
                    seq = self.var_seq[0]
                else:
                    size = len(self.var_seq)
                    seq = self.var_seq
            else:
                if tools.is_homopolymer(self.ref_seq):
                    #print "is homopolymer", self.ref_seq
                    size = 1
                    seq = self.ref_seq[0]
                else:
                    size = len(self.ref_seq)
                    seq = self.ref_seq
                    
            start = int(self.ref_start) - size

	    if size == 0:
		sys.stderr.write("error in upshift size 0 contig:%s %s\n" % (self.var, self.ref_seq))
		return
	    
	    while start > 1:
		upstream = refseq.GetSequence(self.ref, start+1, start+size)
                if seq.upper() != upstream.upper():
                    break


                start = start - size

            if start + size < self.ref_start:
                if self.snv_type == 'ins':
                    sys.stderr.write("shifted %s %s %s %s to %d\n" % (self.var, self.snv_type, self.ref, self.ref_start, start+size))
                    self.ref_start = self.ref_end = start + size
                else:
                    sys.stderr.write("shifted %s %s %s %s to %d\n" % (self.var, self.snv_type, self.ref, self.ref_start, start+size+1))
                    self.ref_start = start + size + 1
                    self.ref_end = self.ref_start + len(self.ref_seq) - 1

    def expand_contig_region(self, contig_sequence, query_strand):
        if self.snv_type != 'ins' and self.snv_type != 'del':
            return None
	
	#don't need to expand contig region if deleted/inserted sequence is longer than contig sequence
	if self.snv_type == 'del' and len(self.ref_seq) > len(contig_sequence):
	    return None
	if self.snv_type == 'ins' and len(self.var_seq) > len(contig_sequence):
	    return None

        if self.snv_type == 'ins':
            seq = self.var_seq[:]
        else:
            seq = self.ref_seq[:]
	    
        if tools.is_homopolymer(seq) or len(seq) == 1:
            homo = True
        else:
            homo = False

        region_before = self.confirm_contig_region[:]
	
	#arbitrary big number
	limit = 100000
	
        #first direction
        expand = 0
        for i in range(limit):
            if homo:
                changed_base = seq[0].upper()
            else:
                changed_base = seq[i%len(seq)].upper()
            downstream_base = None
            #only working
            if self.snv_type == 'del':
                if query_strand == '+':
                    if int(self.var_end) + i < len(contig_sequence):
                        downstream_base = contig_sequence[int(self.var_end)+i].upper()
                        #print "del:down+", query_strand, self.var, i, changed_base, downstream_base, int(self.var_end)+i
                else:
                    if int(self.var_end)-2-i >= 0: 
                        downstream_base = tools.reverse_complement(contig_sequence[int(self.var_end)-2-i]).upper()
                        #print "del:down-", query_strand, self.var, i, changed_base, downstream_base, int(self.var_end)-2-i
            elif self.snv_type == 'ins':
                if query_strand == '+':
                    if int(self.var_end) + i < len(contig_sequence):
                        downstream_base = contig_sequence[int(self.var_end)+i].upper()
                        #print "ins:down+", query_strand, self.var, i, changed_base, downstream_base, int(self.var_end)+i
                else:
                    if int(self.var_start)-i-2 >= 0:
                        downstream_base = tools.reverse_complement(contig_sequence[int(self.var_start)-i-2]).upper()
                        #print "ins:down-", query_strand, self.var, i, changed_base, downstream_base, int(self.var_start)-i-2

            if changed_base == downstream_base:
                expand += 1   
            else:
                break
	
	multiples = expand/len(seq)
	if multiples > 0:
	#if expand > 0 and expand % len(seq) == 0:
            if query_strand == '+':
                self.confirm_contig_region[1] += multiples * self.snv_len
            else:
                self.confirm_contig_region[0] -= multiples * self.snv_len

        #reverse direction
        seq = seq[::-1]
        expand = 0

        for i in range(limit):
            if homo:
                changed_base = seq[0].upper()
            else:
                changed_base = seq[i%len(seq)].upper()
            upstream_base = None
            if self.snv_type == 'del':
                if query_strand == '+':
                    if int(self.var_start)-i-1 >= 0:
                        upstream_base = contig_sequence[int(self.var_start)-i-1].upper()
                        #print "del:up+", query_strand, self.var, i, changed_base, upstream_base, int(self.var_start)-i-1
                else:
                    if int(self.var_start)+i-1 < len(contig_sequence):
                        upstream_base = tools.reverse_complement(contig_sequence[int(self.var_start)+i-1]).upper()
                        #print "del:up-", query_strand, self.var, i, changed_base, upstream_base, int(self.var_start)+i
            elif self.snv_type == 'ins':
                if query_strand == '+':
                    if int(self.var_start)-i-2 >= 0:
                        upstream_base = contig_sequence[int(self.var_start)-i-2].upper()
                        #print "ins:up+", query_strand, self.var, i, changed_base, upstream_base, int(self.var_start)-i-2
                else:
                    if int(self.var_end)+i < len(contig_sequence):
                        upstream_base = tools.reverse_complement(contig_sequence[int(self.var_end)+i]).upper()
                        #print "ins:up-", query_strand, self.var, i, changed_base, upstream_base, int(self.var_end)+i

            if changed_base == upstream_base:
                expand += 1   
            else:
                break

	multiples = expand/len(seq)
	if multiples > 0:
        #if expand > 0 and expand/len(seq) > 0:
            if query_strand == '+':
                self.confirm_contig_region[0] -= multiples * self.snv_len
            else:
                self.confirm_contig_region[1] += multiples * self.snv_len
		
	expanded_sequence = contig_sequence[self.confirm_contig_region[0]-1:self.confirm_contig_region[1]]
	
	#coordinate given in 1-based
        if region_before[0] != self.confirm_contig_region[0] or region_before[1] != self.confirm_contig_region[1]:
	    self.expansion = (self.confirm_contig_region[1] - self.confirm_contig_region[0] + 1) / self.snv_len
            sys.stderr.write("expand confirm contig region %s %s -> %s %s %s %sx\n" % (self.var, region_before, self.confirm_contig_region, expanded_sequence, len(expanded_sequence), self.expansion))
                    
