#!/bin/sh

set -eux

dest=$1
files=$2
genome=$3
codebase=$4

if [ ! -d $dest ]; then
    mkdir $dest
fi

#ftp ucsc
wget -N -nH -x --cut-dirs=3 -P $dest -B ftp://hgdownload.cse.ucsc.edu/goldenPath/$genome/database/ -i $files

cd $dest

#create knownGene_ref.txt
zcat knownGene.txt.gz | sort > tmp1
zcat kgXref.txt.gz | sort > tmp2
join tmp1 tmp2 -t $'\t' > knownGene_ref.txt
rm tmp* knownGene.txt.gz kgXref.txt.gz

#create ensembl_ref.txt
zcat ensGene.txt.gz | sort -k2 > ensGene_sorted.txt
zcat ensemblToGeneName.txt.gz | sort > ensemblToGeneName_sorted.txt
join -1 2 -2 1 ensGene_sorted.txt ensemblToGeneName_sorted.txt -a 1 > ensGene_ref.txt
rm ensGene.txt.gz ensGene_sorted.txt ensemblToGeneName.txt.gz ensemblToGeneName_sorted.txt

#assign aceview gene names
zcat acembly.txt.gz | awk -v FS="[\.\t]" '{print $0"\t"$2}' > acembly_ref.txt
rm acembly.txt.gz

#create repeat coord files for snv code
zcat rmsk.txt.gz | awk '{print $6, $7, $8, $11, $12, $13, $2}' > ${genome}_all_rmsk.coord
zcat simpleRepeat.txt.gz | awk '{print $2,$3,$4,"TRF_SimpleTandemRepeat_"$17}' > ${genome}_simple_repeats.coords
zcat genomicSuperDups.txt.gz | awk '{print $2,$3,$4,$5}' > ${genome}_segdups.coords
rm rmsk.txt.gz simpleRepeat.txt.gz genomicSuperDups.txt.gz

#unzip rest of files
for i in $(/bin/ls *.gz)
    do
        gzip -d $i
done

#index all annotations
source $codebase/setup
file="knownGene_ref.txt"
python $codebase/analysis/annotations/knownGene.py $file -i ${file//txt/idx} -g hg19

file="refGene.txt"
python $codebase/analysis/annotations/refGene.py $file -i ${file//txt/idx} -g hg19

file="ensGene_ref.txt"
python $codebase/analysis/annotations/ensembl.py $file -i ${file//txt/idx} -g hg19

file="acembly_ref.txt"
python $codebase/analysis/annotations/aceview.py $file -i ${file//txt/idx} -g hg19

#index repeats
for i in {segdups,simple_repeats};
    do
        file="${genome}_${i}.coords"
        echo $file
        python $codebase/analysis/annotations/repeat.py $file -i ${file}.idx
done

file="${genome}_all_rmsk.coord"
python $codebase/analysis/annotations/repeatmasker.py $file -i ${file}.idx

cp $codebase/annotations/shared/splice_motifs.txt .