from align_parsers import psl, sam
import sys, os, re, glob
from optparse import OptionParser
import assembly
from track import Track
from utilities  import tools

def parse(file, format, filters=None, splice_motif_file=None, noline=None, refseq=None):
    print file, format
    
    aligns = []
    if format == 'psl':
        aligns = psl.parse(file, filters, splice_motif_file=splice_motif_file, noline=noline, refseq=refseq)
    elif format == 'sam':
        aligns = sam.parse(file, filters, splice_motif_file=splice_motif_file, noline=noline, refseq=refseq, header=True, original_order=True)

    print "total:", len(aligns)
            
    return aligns

def get_targets(file_name):
    "get custom targets"

    targets = []
    f = open(file_name, 'r')
    for line in f.readlines():
        if re.search('\d', line):
            (label, target) = re.split('\t+', line.rstrip('\n').lstrip(' '))
            target = re.sub('(chr|,)', '', target)
            targets.append(target)
    f.close()

    return targets


def output_aligns(outfile, aligns, track_name=None, contigs=None, append=False, header=True, genome=None, refseq=None, color=None, by_fasta=False):
    ext = os.path.splitext(outfile)[1]
    if append:
        out = open(outfile, 'a')
    else:
        out = open(outfile, 'w')

    track_desc = track_name
    #track_name = track_desc

    #alignments - output track
    if aligns:
        if track_name and genome:
            Track.ucsc_targets(genome, aligns)
        
        if header and track_name:
	    header_line = "track name=\"%s\" description=\"%s\" visibility=%d itemRgb=\"On\"" % (track_name, track_desc, 3)
	    if color is not None:
		header_line += ' color=\"%s\"' % (color)
            out.write("%s\n" % (header_line))

        #map contig to contig name
        contig_dict = None
        if contigs:
            contig_dict = dict((contig.num, contig) for contig in contigs)
	    
	#sort alignments by contig order in original input fasta file
	if by_fasta:
	    count = 0
	    ordered = {}
	    for contig in contigs:
		ordered[contig.num] = count
		count += 1
	    aligns.sort(lambda x,y: ordered[x.query] - ordered[y.query])
            
        for align in aligns:
            #this is for adding coverage info
            contig = align.query
            if ' ' in contig:
                contig = align.query.split(" ")[0]
            
            if contig_dict and contig_dict.has_key(contig):
                align.contig = contig_dict[contig]
                                
            if ext == ".gff":
                out.write(align.gff("exon"))

            elif ext == ".psl":
                #ncount = len(re.findall('n', contig_dict[contig].sequence, re.IGNORECASE))
                out.write(align.psl(refseq=refseq))

            elif ext == ".exon":
                out.write(align.exon())
                
    out.close()


def get_contigs(fasta, kmer=None, lib=None):
    sys.stderr.write("extracting contigs...")

    if kmer:
        ass = assembly.Assembly(lib, k=kmer)
    else:
        ass = assembly.Assembly(lib, k=1)
        
    ass.fasta = fasta
    contigs = ass.get_contigs()

    for contig in contigs:
        if ":" in contig.num:
            k, ctg = contig.num.split(':')
            contig.k = k.lstrip('k')
        elif kmer:
            contig.k = kmer

    sys.stderr.write("done\n")
    return contigs


def batch(format, indir, ext=None):
    inputs = glob.glob(os.path.join(indir + '/*.'+'[0-9]*.' + format))
    return sorted(inputs)


if __name__ == '__main__':
    usage = "Usage: %prog file format [options]"
    parser = OptionParser(usage=usage)
    parser.add_option("-n", "--bestn", dest="bestn", help="best hit filter")
    parser.add_option("-i", "--identity", dest="identity", help="identity filter")
    parser.add_option("-m", "--match", dest="match_percent", help="match length percentage")
    parser.add_option("-u", "--unique", dest="unique", help="unique", action="store_true", default=False)
    parser.add_option("-q", "--query_len", dest="query_len", help="query length filter")
    parser.add_option("-t", "--target", dest="target", help="target e.g. chr1:35422000-35425800, chr13, or file containing targets")
    parser.add_option("-o", "--out", dest="out", help="output file, with proper file extension e.g. x.gff, x.psl, x.snv, or x.gff,x.snv")
    parser.add_option("-k", "--track", dest="track_name", help="track name e.g. \"HS1202 k45\"")
    parser.add_option("-f", "--fasta", dest="fasta", help="fasta file, for extracting contig info")
    parser.add_option("-K", "--k", dest="k", help="kmer value, if not present in contig name")
    parser.add_option("-d", "--batch", dest="batch", help="batch mode", action="store_true", default=False)
    parser.add_option("-g", "--genome", dest="genome", help="genome for UCSC chromosome conversion")
    parser.add_option("-r", "--ref", dest="ref", help="use reference sequence for deducing number of Ns in genomic sequence", action="store_true", default=False)
    parser.add_option("-c", "--color", dest="color", help="RGB color for track e.g. 255,0,0")
    parser.add_option("-z", "--zip", dest="gzip", help="gzip track", action="store_true", default=False)
    parser.add_option("-s", "--by_fasta", dest="by_fasta", help="order output by fasta inputs. Default: original alignment order", action="store_true", default=False)
    
    (options, args) = parser.parse_args()
    
    filters = {}
    if len(args) == 2:
        if options.bestn:
            filters['bestn'] = int(options.bestn)
        if options.identity:
            filters['identity'] = float(options.identity)
        if options.match_percent:
            filters['match'] = float(options.match_percent)
        if options.unique:
            filters['unique'] = True
        if options.query_len:
            filters['qlen'] = int(options.query_len)
        if options.target:
            if os.path.exists(options.target):
                filters['target'] = get_targets(options.target)
            else:
                filters['target'] = [options.target]

        #get contig/coverage
        contigs = None
        if options.fasta:
            contigs = get_contigs(options.fasta, options.k)
            
        refseq = None
	if options.ref:
	    refseq = tools.get_refseq_from_2bit(options.genome)

        #inputs
        inputs = []
        if options.batch:
            inputs = batch(args[1], args[0])
        else:
            inputs.append(args[0])

        if options.batch:
            if options.out and os.path.exists(options.out):
                os.remove(options.out)
                
        for i in range(len(inputs)):
            infile = inputs[i]
            #for deciding if header needs to be output
            first = False
            if i == 0:
                first = True

	    aligns = parse(infile, args[1], filters)
            
            if options.out:
                output_aligns(options.out, aligns, options.track_name, contigs=contigs, 
		              append=options.batch, header=first, 
		              genome=options.genome, refseq=refseq, 
		              color=options.color, by_fasta=options.by_fasta)

	if options.gzip:
	    os.system('gzip ' + options.out)
	    
    else:
        parser.error("incorrect number of arguments")
