from optparse import OptionParser
from intspan import overlap, subsume
import os, sys, re
from tools import reverse_complement, seq_overlap
import pysam

class BAM:

    bam = None
    reads = None
    min_mapq = 1

    def __init__(self, bam_file, min_mapq=1, sample_type=None):
	self.source = bam_file
        self.bam = pysam.Samfile(bam_file, "rb")
        self.min_mapq = min_mapq
	self.sample_type = sample_type

    def confirm(self, target, pos=[], feature=None, allele=None, regions=None, seq=False, breakpoint=None, breakpoint_buffer=0, refseq=None):
        feature = feature.lower()
        reads = []

        if target and not 'chr' in self.bam.references[0]:
            target = target.replace('chr', '')
	    
	if target and target[0] == 'k' and '_' in target and not '_' in self.bam.references[0] and ':' in self.bam.references[0]:
	    target = target.replace('_', ':')

        if target and target not in self.bam.references:
            if seq:
                return []
            else:
                return 0
        
        if feature == 'snv' or feature == 'inv':
            reads = self.confirm_snv(target, pos[0], allele, seq=seq)
        elif feature == 'del':
            reads = self.confirm_del(target, pos, seq=seq, refseq=refseq)
	    
	#special case: confirm_ins only returns number
        elif feature == 'ins':
            reads = self.confirm_ins(target, pos[0], allele, refseq=refseq)
	    
        elif feature == 'breakpoint':
            if not regions:
                reads = self.confirm_breakpoint_contig(target, pos, seq=seq)
            else:
                reads = self.confirm_breakpoint_genomic(regions, seq=seq, breakpoint=breakpoint, breakpoint_buffer=breakpoint_buffer)
		
        return reads

    def confirm_perfect(self, target, pos, allele, strand, seq=False, min_from_end=0, expansion=0):
        '''
        confirm every base in the target is the same as allele
        '''
	
	if target and 'chr' in target and not 'chr' in self.bam.references[0]:
            target = target.replace('chr', '')
	    
	if target and target not in self.bam.references:
            if seq:
                return []
            else:
                return 0
	
	#needed because pos may be extracted from text file
	pos[0] = int(pos[0])
	pos[1] = int(pos[1])
	
	pos[0] = max(pos[0], 1)
        num_reads = 0
        reads = []
	reads_dict = {}
	
	if expansion > 1:
	    i = 1;
	    seq = allele
	    while i < expansion:
		allele = allele + seq
		i = i+1
	
	#for insertion longer than read length
	coverage = {}
	for i in range(pos[0], pos[1]+1):
	    coverage[i] = 0
	
	shorter_than_read = False
        for read in self.bam.fetch(target, pos[0], pos[1] + 1):
	    if len(allele) <= read.rlen:
		shorter_than_read = True
		if read.pos+1 <= pos[0] and read.pos+read.rlen >= pos[1]:
		    status = "unknown"
		    from_end = min(pos[0] - read.pos -1, read.pos + read.rlen - pos[1])
		    offset = pos[0]-read.pos-1
		    bases = read.seq[offset:offset+len(allele)]
		
		    #only needs to rc bases if contig is aligned to negative strand, as read-sequence(reported) is same as contig sequence    
		    if strand == '-':
			bases = reverse_complement(bases)
		
		    if from_end < min_from_end:
			status = "fail_edge"
		    
		    elif bases.lower() == allele.lower():
			status = "perfect"
			if seq:
			    reads.append(read)
			num_reads += 1
		    
		    else:
			status = "non-identical_base"
		    			
	    
	#none of the reads is longer than the allele(insertion), or no read within pos
	if not shorter_than_read:
	    allele_seq = allele
	    if strand == '-':
		allele_seq = reverse_complement(allele)
		
	    for pileupcolumn in self.bam.pileup(target, pos[0]-1, pos[1]):
		if shorter_than_read:
		    break
		
		if pileupcolumn.pos >= pos[0]-1 and pileupcolumn.pos <= pos[1]-1:
		    cov = pileupcolumn.n
		    
		    for pileupread in pileupcolumn.pileups:
			#allele not really longer than read length
			if len(allele) <= len(pileupread.alignment.seq):
			    shorter_than_read = True
			    break
			    			
			#pysam bug?
			if pileupread.qpos >= len(pileupread.alignment.seq):
			    continue
						
			if pileupread.alignment.seq[pileupread.qpos].lower() != allele_seq[pileupcolumn.pos-pos[0]+1]:
			    cov = cov - 1
			else:
			    reads_dict[pileupread.alignment.qname] = pileupread.alignment
		    coverage[pileupcolumn.pos+1] = cov
		    			    	    
	    #determine if allele is entirely covered
	    gaps = False
	    for i in range(pos[0], pos[1]+1):
		if coverage[i] == 0:
		    gaps = True
		    break
		
	    if not gaps:
		if seq:
		    reads = reads_dict.values()
		else:
		    num_reads = len(reads_dict.keys())
	
        if seq:
            return reads
        else:
            return num_reads
                               

    def confirm_snv(self, target, pos, allele, min_from_end=8, seq=False, max_depth=2000, max_match=1000):        
	num_reads = 0
	reads = []

        if not allele or len(allele) < 1:
	    if seq:
		return None
	    else:
		return 0
			    
	count = 0
	for read in self.bam.fetch(target, pos, pos + 1):
	    #no duplicate-read allowed
	    if read.is_duplicate:
		continue
	    
	    #filter out chastity-failed reads
	    if read.flag & 512 != 0:
		continue
	    
	    base = self.get_aligned_base(read, pos)
	    
	    if base and base.lower() == allele.lower():
		from_end = pos - read.pos - 1
		if read.rlen - from_end < from_end:
		    from_end = read.pos + read.rlen - pos
		    		
		if from_end >= min_from_end:
		    if seq:
			reads.append(read)
		    else:
			num_reads += 1
						
	    if num_reads == max_match or len(reads) == max_match:
		break
	    
	    count += 1
	    if count == max_depth:
		break
	    
	if not seq:
	    return num_reads
        else:
	    return reads
	
	
    def get_aligned_base(self, read, target_pos):
	bases = {}
	start = read.pos
	query_count = 0
	target_count = read.pos

	if read.cigar is None:
	    return None
	
	for i in range(len(read.cigar)):
	    op, size = read.cigar[i]
	    
	    #start is soft-clipped
	    if i == 0 and op == 4:
		query_count += size
		continue
		
	    #match
	    if op == 0:
		for j in range(size):
		    bases[target_count + j] = read.seq[query_count + j]
		    
		target_count += size
		query_count += size
		
	    #insertion
	    elif op == 1:
		query_count += size
	    
	    #deletion
	    elif op == 2:
		target_count += size
		   
	if bases.has_key(target_pos - 1):
	    return bases[target_pos - 1]
	else:
	    return None
	    
	
    def confirm_del(self, target, pos, refseq=None, fragment_buffer=1000, seq=False):
	"""
	Will extract all reads containing the deletion
	If the deletion size is the same but has different coordinate in the bam file,
	will compare the sequence after deletion to see if the read contains the same event
	"""
	size = pos[1] - pos[0] + 1
			
	good_reads = []
	bigger_than_rlen = False
	for read in self.bam.fetch(target, pos[0], pos[1] + 1):
	    if int(read.mapq) < int(self.min_mapq):
                continue
	    	    
	    if pos[1] - pos[0] + 1 > read.rlen:
		bigger_than_rlen = True
		break
	    
	    if read.pos+1 <= pos[0] and read.pos + self.aligned_len(read) >= pos[1]:
		align = self.parse_cigar(read)
		dels = [a for a in align if a[0] == 'del' and a[2] - a[1] + 1 == size]
		
		#if there is a deletion of the same size identified from the cigar string
		if dels:		    		    
		    expected_seq = refseq.GetSequence(target, read.pos + 1, pos[0] - 1) + refseq.GetSequence(target, pos[1] + 1, read.pos + max(read.rlen, self.aligned_len(read)) + size)
		    
		    for d in dels:
			if read.pos+1 > d[0] or read.pos + self.aligned_len(read) < d[1]:
			    continue
			
			#if the start and end of the deletion match, good
			if d[1] == pos[0] and d[2] == pos[1]:
			    good_reads.append(read)
			    break
			
			read_seq = refseq.GetSequence(target, read.pos + 1, d[1] - 1) + refseq.GetSequence(target, d[2] + 1, read.pos + max(read.rlen, self.aligned_len(read)) + size)
			if read_seq.lower() == expected_seq.lower():
			    good_reads.append(read)
			    break
			    
	if bigger_than_rlen:
	    genome_buffer = 2 * fragment_buffer
	    regions = [[target, pos[0] - genome_buffer, pos[0]], [target, pos[1], pos[1] + genome_buffer]]
	    breakpoint = [[target, pos[0]], [target, pos[1]]]
	                  
	    good_reads = self.confirm_breakpoint_genomic(regions, breakpoint=breakpoint, seq=True)
		
	if not seq:
	    return len(good_reads)
	else:
	    return good_reads
		
    def parse_cigar(self, read):
	"""
	Reports cigar string alignment in genomic coordinate
	For use in confirm_ins() and confirm_del()
	"""
	align = []
	
	end = read.pos
	for i in range(len(read.cigar)):				    
	    if read.cigar[i][0] == 0:
		align_type = 'match'		
		start = end + 1
		end = start + read.cigar[i][1] - 1
		align.append([align_type, start, end])
		
	    elif read.cigar[i][0] == 1:
		align_type = 'ins'		
		align.append([align_type, end, read.cigar[i][1]])
		
	    elif read.cigar[i][0] == 2:
		align_type = 'del'
		start = end + 1
		end = start + read.cigar[i][1] - 1	
		align.append([align_type, start, end])
		
	    elif read.cigar[i][0] == 4:
		align_type = 'softclipped'
		start = end + 1
		end = start + read.cigar[i][1] - 1	
		align.append([align_type, start, end])
		
	return align
    
	
    def aligned_len(self, read):
	"""
	Returns aligned length from cigar string
	"""
	
	if read.cigar:
	    #don't add deletion length(2), intron length(3), silent deletion(6)
	    lens = [a[1] for a in read.cigar if a[0] != 2 and a[0] != 3 and a[0] != 6]
	    return sum(lens)
	else:
	    return read.rlen
		
    
    def confirm_ins(self, target, pos, allele, refseq=None, fragment_buffer=1000, big_cutoff=50):
	"""
	Will extract all reads containing the given insertion
	If the insertion size is the same but has different coordinate in the bam file,
	will compare the sequence after insertion to see if the read contains the same event
	"""
        size = len(allele)
			
	good_reads = []
	num_reads_extracted = 0
	bigger_than_rlen = False
	
	for read in self.bam.fetch(target, pos, pos + 1):
	    num_reads_extracted += 1
	    
	    if len(allele) > read.rlen:
		bigger_than_rlen = True
		break
	    
	    if int(read.mapq) < int(self.min_mapq):
                continue
	    
            if read.pos+1 <= pos and read.pos + read.rlen >= pos:
		align = self.parse_cigar(read)
		ins = [a for a in align if a[0] == 'ins' and a[2] == size]
		
		#if there is an insertion of the same size identified from the cigar string
		if ins:
		    expected_seq = refseq.GetSequence(target, read.pos + 1, pos) + allele + refseq.GetSequence(target, pos + 1, read.pos + max(read.rlen, self.aligned_len(read)) - size)
		    
		    for i in ins:
			if read.pos+1 > i[0] or read.pos + self.aligned_len(read) < i[1]:
			    continue
			
			read_ins_start = i[1] - read.pos
			read_allele = read.seq[read_ins_start:read_ins_start + i[2]]
			    
			#if position is the same and allele is the same, good
			if i[1] == pos and read_allele.lower() == allele.lower():
			    good_reads.append(read)
			    break
			
			read_seq = refseq.GetSequence(target, read.pos + 1, i[1]) + read_allele + refseq.GetSequence(target, i[1]+1, read.pos + max(read.rlen, self.aligned_len(read)) - size)
			    
			if read_seq.lower() == expected_seq.lower():
			    good_reads.append(read)
			    break
	
	if bigger_than_rlen or (num_reads_extracted == 0 and len(allele) > big_cutoff):
	    genome_buffer = 2 * fragment_buffer
	             
	    num_pairs_novel_upstream = self.confirm_novel((target, max(1, pos - genome_buffer), pos), allele, breakpoint=pos)
	    num_pairs_novel_downstream = self.confirm_novel((target, pos + 1, pos + 1 + genome_buffer), allele, breakpoint=pos + 1)
	    num_clipped_reads = self.confirm_clipped_reads((target, pos - genome_buffer, pos + genome_buffer), breakpoint=pos, novel_seq=allele)
	    
	    num_reads = num_clipped_reads + min(num_pairs_novel_upstream, num_pairs_novel_downstream) 
	    	    
	else:
	    num_reads = len(good_reads)
	        
	return num_reads
	
    def confirm_novel(self, region, novel_seq, breakpoint, min_olap=10, max_unmapped=1000):
	'''
	Given novel sequence, find read-pairs that have one unmapped mate that overlaps (with minimum overlap) with novel sequence
	'''
	
	unmapped = {}
	count = 0
	for read in self.bam.fetch(region[0], region[1], region[2]):
	    if read.is_paired and read.mate_is_unmapped:
		count += 1
		unmapped[read.qname] = read
		
		if count == max_unmapped:
		    break
		

	mates = 0
	found_mates = 0
	good_reads = []
		
	if unmapped:
	    mate_seq = self.get_unmapped_mate_seq(region[0], unmapped.values())
	    
	    for read in unmapped.values():
		if (breakpoint == region[1] and not read.is_reverse) or \
		   (breakpoint == region[2] and read.is_reverse):
		    continue		
		
		if mate_seq.has_key(read.qname):
		    if read.pos > region[1]:
			from_end = read.pos - region[1] + 1
		    else:
			from_end = region[1] - read.pos + 1
			
		    mates += 1
		    
		    m = re.search(mate_seq[read.qname], novel_seq, re.IGNORECASE)
		    if m:
			found_mates += 1
			
		    else:
			m = re.search(reverse_complement(mate_seq[read.qname]), novel_seq, re.IGNORECASE)
			if m:
			    found_mates += 1
			    
			else:
			    olap = seq_overlap(mate_seq[read.qname], novel_seq)
			    if olap > min_olap:
				found_mates += 1
				
			    else:
				olap = seq_overlap(reverse_complement(mate_seq[read.qname]), novel_seq)
				if olap > min_olap:
				    found_mates += 1
				    			    
	return found_mates
	    
    def confirm_clipped_reads(self, region, breakpoint, novel_seq, min_clipped=5):
	num_clipped_reads = 0
	
	for read in self.bam.fetch(region[0], region[1], region[2]):
	    clipped_seq = self.get_clipped_seq(read, breakpoint)
	    
	    if clipped_seq and len(clipped_seq[1]) > min_clipped:
		clipped_seq_matched = False
		
		if clipped_seq[0] == 'start':
		    olap = seq_overlap(novel_seq, read.seq)
		    
		    if olap > len(clipped_seq):
			clipped_seq_matched = True
		    
		else:
		    olap = seq_overlap(read.seq, novel_seq)
		    
		    if olap > len(clipped_seq):
			clipped_seq_matched = True
			
		if clipped_seq_matched:
		    num_clipped_reads += 1
		
	return num_clipped_reads

    
    def get_unmapped_mate_seq(self, target, reads, max_reads=1000):
	'''
	Given a list of reads that don't have mates mapped, extract the sequences of their unmapped mates
	Must NOT be run within another loop of iteration of read extraction
	'''
	seq = {}
	for unmapped_read in reads:
	    for read in self.bam.fetch(target, max(1, unmapped_read.pos - unmapped_read.rlen), unmapped_read.pos + unmapped_read.rlen):
		if read.qname == unmapped_read.qname and read.is_read1 != unmapped_read.is_read1:
		    seq[read.qname] = read.seq
        
	return seq
    
    def is_soft_clipped(self, read, breakpoint):
	'''
	Given read, determine if it's soft-clipped from cigar
	'''
	if read.cigar and (read.cigar[0][0] == 4 or read.cigar[-1][0] == 4):
	    #soft-clipped at beginning
	    if read.cigar[0][0] == 4:
		start = read.pos - read.cigar[0][1]
	    else:
		start = read.pos
		
	    end = start + self.aligned_len(read)
	    
	    if breakpoint >= start and breakpoint <= end:
		return True
	    
	
	return False
    
    def get_clipped_seq(self, read, breakpoint):
	if read.cigar and (read.cigar[0][0] == 4 or read.cigar[-1][0] == 4):
	    if read.cigar[0][0] == 4:
		start = read.pos - read.cigar[0][1]
	    else:
		start = read.pos
		
	    end = start + self.aligned_len(read)
	    
	    #make sure soft-clipped reads overlap with breakpoint
	    if breakpoint >= start and breakpoint <= end:
		if read.cigar[0][0] == 4:
		    return ('start', read.seq[:read.cigar[0][1]])
		else:
		    return ('end', read.seq[-1 * read.cigar[-1][1]:])
	    
	return None
    
    def confirm_breakpoint_contig(self, target, pos, seq=False):
        num_reads = 0
        reads = {}

        pos[0] = max(pos[0], 1)

        for read in self.bam.fetch(target, pos[0], pos[1] + 1):
            if int(read.mapq) < int(self.min_mapq) or read.is_unmapped:
                continue

            if read.pos+1 <= pos[0] and read.pos + read.rlen >= pos[1]:
		reads[read.qname] = read
		    
            elif read.rlen < (pos[1] - pos[0] + 1):
                reads[read.qname] = read

        if not seq:
            return len(reads.keys())
        else:
            return reads.values()
    
		
    def check_genome_region(self, region):
	#get rid of 'chr' if bam file's genome doesn't have chr
        if not 'chr' in self.bam.references[0] and 'chr' in region[0]:
            region[0] = region[0].replace('chr', '')
	    
	#make sure start > 0
	region[1] = max(1, region[1])
	
        if not region[0] in self.bam.references:
	    return False
	
        #don't do whole chromosome comparisons
        if int(region[1]) == 0 and int(region[2]) == 0:
	    return False
	
	return True
	
    
    def confirm_breakpoint_genomic(self, regions, max_pairs=10000, breakpoint=None, seq=False, breakpoint_buffer=0):
	if not self.check_genome_region(regions[0]) or not self.check_genome_region(regions[1]):
	    if not seq:
		return 0
	    else:
		return None
			
        num_reads = 0
        region1 = regions[0]
        region2 = regions[1]

	reads1 = self.find_mates_in_genome(region1, breakpoint[0], breakpoint_buffer=breakpoint_buffer, outside_breakpoint=True)
	reads1_dict = dict((r.qname, r) for r in reads1)
	reads1_pos = {}
	for read in reads1:
	    reads1_pos[read.qname + ' ' + str(read.pos)] = True
	
	reads2 = self.find_mates_in_genome(region2, breakpoint[1], breakpoint_buffer=breakpoint_buffer, outside_breakpoint=True)
	reads2_dict = dict((r.qname, r) for r in reads2)
	reads2_pos = {}
	for read in reads2:
	    reads2_pos[read.qname + ' ' + str(read.pos)] = True		    
	    
	#find real pairs
	pairs = {}
	for read in reads1:
	    if reads2_dict.has_key(read.qname):
		pairs[read.qname] = read, reads2_dict[read.qname]
		
	#calculate stacking
	stacks1 = {}
	stacks2 = {}
	for name, reads in pairs.iteritems():
	    if not stacks1.has_key(reads[0].pos):
		stacks1[reads[0].pos] = 0
	    stacks1[reads[0].pos] += 1
	    
	    if not stacks2.has_key(reads[1].pos):
		stacks2[reads[1].pos] = 0
	    stacks2[reads[1].pos] += 1
		
	num_reads = len(pairs.keys())
			    
	#rejects reads that are stacking
	if (len(stacks1.keys()) == 1 and stacks1[stacks1.keys()[0]] > 1) or (len(stacks2.keys()) == 1 and stacks2[stacks2.keys()[0]] > 1):
	    num_reads = num_reads * -1
	   
	if not seq:
	    return num_reads
	else:
	    return pairs.values()
	
	
    def find_mates_in_genome(self, region, breakpoint, breakpoint_buffer=0, missing_mates=None, outside_breakpoint=True):
	if not self.check_genome_region(region):
	    return []
		    
	reads = []
	for read in self.bam.fetch(region[0], region[1], region[2]):
	    if int(read.mapq) < int(self.min_mapq):
		continue
	    
	    if missing_mates and not self.is_mate(read, missing_mates):
		continue
	    
	    #filter out chastity-failed reads
	    if read.flag & 512 != 0:
		continue
	    
	    alen = self.aligned_len(read)

	    pointing_correctly = False
	    completely_on_one_side = False
	    #sense
	    if not read.is_reverse:
		if read.pos < breakpoint[1]:
		    pointing_correctly = True
		
		    if read.pos + alen - 1 <= breakpoint[1] + breakpoint_buffer:
			completely_on_one_side = True
						
	    #anti-sense
	    elif read.pos + alen - 1 > breakpoint[1]:
		pointing_correctly = True
		
		if read.pos >= breakpoint[1] - breakpoint_buffer:
		    completely_on_one_side = True
		    			
	    if pointing_correctly:		
		if outside_breakpoint:
		    if completely_on_one_side:
			reads.append(read)
		else:
		    reads.append(read)
		
	return reads
   		
	
    def is_single_stack(self, reads):
	stacks = {}
	for read in reads:
	    if not stacks.has_key(read.pos):
		stacks[read.pos] = []
	    stacks[read.pos].append(read)
	    
	if len(reads) > 1 and len(stacks.keys()) == 1:
	    return True
	
	else:
	    return False
	
	
    def is_mate(self, read, mates):
	'''
	Given a read and a dictionary of mates (key:read.qname, value: read),
	see if read is mate of one of the mates
	'''
	
	if mates.has_key(read.qname):
	    mate = mates[read.qname]
    
	    if read.is_read1 and not mate.is_read1 or not read.is_read1 and mate.is_read1:
		return mate
		
	return False
    	
	    
    def coverage_simple(self, target, pos=[], total=False, seq=False):
        ''' not counting split reads '''
        
        if not 'chr' in self.bam.references[0] and 'chr' in target:
            target = target.replace('chr', '')

        if not target in self.bam.references:
            if seq:
                return []
            else:
                return 0

        coverage_profile = {}
        for i in range(pos[0],pos[1]+1):
            coverage_profile[i] = 0

        reads = []
        num_reads = 0
        for read in self.bam.fetch(target, pos[0], pos[1]):
            read_end = read.pos + read.rlen - 1
            if read.pos+1 <= pos[1] and read_end+1 >= pos[0]:
                num_reads += 1
                
                if seq:
                    reads.append(read)
                    
                for i in range(max(read.pos, pos[0]-1), min(read_end, pos[1]-1)+1):
                    coverage_profile[i+1] += 1

        if not total:
            return "%.1f" % (sum(coverage_profile.values()) / len(coverage_profile.keys()))
        elif seq:
            return reads
        else:
            return num_reads

    def coverage_simple_pileup(self, target, pos=[], max_depth=10000):
        ''' use pileup, will count split reads '''
        
        if not 'chr' in self.bam.references[0] and 'chr' in target:
            target = target.replace('chr', '')
        
        if not target in self.bam.references:
            return None
        
        pair_used = {}
        coverage_profile = {}
        for i in range(pos[0],pos[1]+1):
            coverage_profile[i] = 0

        for pileupcolumn in self.bam.pileup(target, pos[0], pos[1]):
            if pileupcolumn.n >= max_depth:
                return None
            
            if pileupcolumn.pos >= pos[0]-1 and pileupcolumn.pos <= pos[1]-1:
                coverage_profile[pileupcolumn.pos+1] = pileupcolumn.n

        return "%.1f" % (sum(coverage_profile.values()) / len(coverage_profile.keys()))
                
    def coverage(self, target, pos=[], read=True, fragment=False, profile=False, use_orphan=True, insert_size=0):
        if not 'chr' in self.bam.references[0] and 'chr' in target:
            target = target.replace('chr', '')
        
        if not target in self.bam.references:
            return None
        
        pair_used = {}
        coverage_profile = {}
        for i in range(pos[0],pos[1]+1):
            coverage_profile[i] = 0

        pileup_start = pos[0]
        pileup_end = pos[1]
        if insert_size and insert_size > pos[1]-pos[0]:
            pileup_start = pos[1] - insert_size
            pileup_end = pos[0] + insert_size
            if pileup_start < 0:
                pileup_start = 0
                
        for pileupcolumn in self.bam.pileup(target, pileup_start, pileup_end):
            if pileupcolumn.pos >= pileup_start-1 and pileupcolumn.pos <= pileup_end-1:
                for pileupread in pileupcolumn.pileups:
                    if fragment:
                        if pileupread.alignment.is_proper_pair:
                            if not pair_used.has_key(pileupread.alignment.qname):
                                if pileupread.alignment.pos < pileupread.alignment.mpos:
                                    start = pileupread.alignment.pos+1
                                    end = pileupread.alignment.mpos+1+pileupread.alignment.rlen-1
                                else:
                                    end = pileupread.alignment.pos+1+pileupread.alignment.rlen-1
                                    start = pileupread.alignment.mpos+1
                                    
                                for i in range(start, end+1):
                                    if i >= pos[0] and i <= pos[1]:
                                        coverage_profile[i] += 1

                                orient = '+'
                                if pileupread.alignment.is_reverse:
                                    orient = '-'
                                mate_orient = '+'
                                if pileupread.alignment.mate_is_reverse:
                                    mate_orient = '-'
                                pair_used[pileupread.alignment.qname] = True
                        elif use_orphan:
                            if pileupcolumn.pos >= pos[0]-1 and pileupcolumn.pos <= pos[1]-1:
                                coverage_profile[pileupcolumn.pos+1] += 1
                    elif pileupcolumn.pos >= pos[0]-1 and pileupcolumn.pos <= pos[1]-1:
                        if use_orphan:
                            coverage_profile[pileupcolumn.pos+1] += 1
                        else:
                            if pileupread.alignment.is_proper_pair:
                                coverage_profile[pileupcolumn.pos+1] += 1
                            
        if profile:
            return coverage_profile
        else:
            return "%.1f" % (sum(coverage_profile.values()) / (pileup_end-pileup_start+1))


    def get_coverage(self, target, pos=[]):
	if pos:
	    start, end = pos[0], pos[1]
	else:
	    start, end = None, None
	    
	count = 0
	num_bases = 0
	for read in self.bam.fetch(target, start, end):
	    count += 1
	    num_bases += int(read.rlen)
	    
	return count, num_bases
	    
	    
def get_reads(bam, target, pos=[], feature=None, allele=None):
    "get reads covering specified region"

    reads = []
    
    if feature.lower() == 'snv' and allele:
        for pileupcolumn in bam.pileup(target, pos[0], pos[0]):
            if pileupcolumn.pos == pos[0]-1:
                for pileupread in pileupcolumn.pileups:
                    read = {}

                    read['name'] = pileupread.alignment.qname
                    read['from_end'] = None
                    read_len = pileupread.alignment.rlen
                    if read_len == 0:
                        read_len = len(pileupread.alignment.seq)
                    read['from_end'] = pileupread.qpos
                    if read_len - pileupread.qpos < read['from_end']:
                            read['from_end'] = read_len - pileupread.qpos - 1
                    
                    if pileupread.alignment.seq[pileupread.qpos].lower() == allele.lower():
                        read['support'] = True
                    else:
                        read['support'] = False

                    reads.append(read)
                    
    elif feature.lower() == 'del':
        reads_dict = {}
        
        for pileupcolumn in bam.pileup(target, pos[0], pos[1]):
            if pileupcolumn.pos >= pos[0]-1 and pileupcolumn.pos <= pos[1]-1:
                for pileupread in pileupcolumn.pileups:
                    if reads_dict.has_key(pileupread.alignment.qname):
                        read = reads_dict[pileupread.alignment.qname]
                    else:
                        read = {}
                        read['name'] = pileupread.alignment.qname
                        read['support'] = True
                        
                        read_len = pileupread.alignment.rlen
                        if read_len == 0:
                            read_len = len(pileupread.alignment.seq)
                        from_end = pileupread.qpos
                        if read_len - pileupread.qpos < from_end:
                            from_end = read_len - pileupread.qpos - pileupread.indel
                        if not read.has_key('from_end') or from_end < read['from_end']:
                            read['from_end'] = from_end
                            

                        reads_dict[pileupread.alignment.qname] = read
                        
                    if pileupread.is_del < 1:
                        read['support'] = False

        reads = reads_dict.values()
            
                    

    elif feature.lower() == 'ins':
        for pileupcolumn in bam.pileup(target, pos[0], pos[0]):
            if pileupcolumn.pos == pos[0]-1:
                for pileupread in pileupcolumn.pileups:
                    read = {}
                    
                    read['name'] = pileupread.alignment.qname
                    read['from_end'] = None
                    read_len = pileupread.alignment.rlen
                    if read_len == 0:
                        read_len = len(pileupread.alignment.seq)
                    
                    if pileupread.indel > 0:
                        read['support'] = True
                        read['bases'] = pileupread.alignment.seq[pileupread.qpos+1:pileupread.qpos+1+pileupread.indel]
                        read['from_end'] = pileupread.qpos + 1
                        if read_len - pileupread.qpos < read['from_end']:
                            read['from_end'] = read_len - pileupread.qpos - pileupread.indel - 1
                    else:
                        read['support'] = False
                        read['bases'] = None

                    reads.append(read)


if __name__ == '__main__':
    usage = "Usage: %proj bam_file"

    parser = OptionParser(usage=usage)
    parser.add_option("-f", "--feature", dest="feature", help="feature: snv,ins,del,breakpoint")
    parser.add_option("-t", "--target", dest="target", help="target, chr or contig number")
    parser.add_option("-s", "--start", dest="start", help="start coordinate", type='int')
    parser.add_option("-e", "--end", dest="end", help="end coordinate", type='int')
    parser.add_option("-a", "--allele", dest="allele", help="allele: base(s) for snv or insertion")
    parser.add_option("-c", "--coverage", dest="coverage", help="coverage", action="store_true", default=False)
    parser.add_option("-r", "--region1", dest="region1", help="region1 (breakpoint)", nargs=3)
    parser.add_option("-R", "--region2", dest="region2", help="region2 (breakpoint)", nargs=3)
    
    (options, args) = parser.parse_args()

    bam_file = args[0]
    if len(args) == 1 and os.path.exists(bam_file):
        bam = BAM(bam_file)

        if options.coverage:
            coverage = bam.coverage(options.target, (options.start, options.end))

	else:
            regions = None
            if options.region1:
                regions = [options.region1, options.region2]
            reads = bam.confirm(options.target, (options.start, options.end), feature=options.feature, allele=options.allele, regions=regions)

            for i in range(len(reads)):
                read = reads[i]
                output = []
                for attr in sorted(read.keys()):
                    output.append("%s:%s" % (attr, read[attr]))
		    
    else:
        parser.error("cannot find given bam file")

        
