import sys, glob, os, re, subprocess, shutil
import cfg
from optparse import OptionParser

class Cluster:
    def __init__(self, head_node):
        self.head_node = head_node
        
    def create_job(self, job_file, cmds, name=None, queue='all.q', mem='1G', working_dir=None, depend=[], ncpus=1, excl=False, email=None):
        if ',' in mem:
            mem, ncpus = mem.split(',')
        
        out = open(job_file, 'w')
        out.write("#! /bin/bash\n")
        out.write("#$ -S /bin/bash\n")
        out.write("#$ -R y\n")
        out.write("#$ -q %s\n" % (queue))
        out.write("#$ -l mem_free=%s -l mem_token=%s -l h_vmem=%s\n" % (mem, mem, mem))
        
        if name:
            out.write("#$ -N %s\n" % (name))
        if working_dir:
            out.write("#$ -wd %s\n" % (working_dir))
        if ncpus is not None and ncpus > 1:
            out.write("#$ -pe ncpus %s\n" % (ncpus))
        if excl:
            out.write("#$ -l excl=true\n")
        if depend:
            out.write("#$ -hold_jid %s\n" % (','.join([str(jid) for jid in depend])))
        if email:
            out.write("#$ -M %s\n#$ -m as\n" % (email))
            
        out.write('\n')
        for cmd in cmds:
            out.write(cmd + '\n')
        out.close()
        
        return True
        
        
    def submit_job(self, job_file):
        '''
        qsub jobs (code modifified from Anchor)
        '''
        
        sys.stderr.write('submitting %s to %s\n' % (job_file, self.head_node))
                
        proc = subprocess.Popen(
            ['ssh', self.head_node, 'qsub', job_file],
            shell=False, cwd=os.getcwd(), stderr=subprocess.PIPE,
            stdout=subprocess.PIPE,
            )
        proc.wait()
        
        submit_stdout = proc.stdout.readlines()
        submit_stderr = proc.stderr.readlines()
        
        if submit_stdout:
            for mess in submit_stdout:
                print mess

        if submit_stderr:
            for mess in submit_stderr:
                print mess
        
        job_id = None
        if "Your job" in submit_stdout[0]:
            job_id = int(submit_stdout[0].split()[2])

        return job_id
        
        

PACKAGE_DIR = "/".join(os.path.abspath(sys.argv[0]).split("/")[:-2])
SOURCE = "source " + PACKAGE_DIR + "/setup"
SUBMIT_SCRIPT = PACKAGE_DIR + "/utilities/submitjobs.sh"

def create_jobs_file(input_dir, command, ext, out, jobs_file):
    output = open(jobs_file, 'w')
    print input_dir, ext
    for infile in glob.glob(os.path.join(input_dir, '*'+ext)):
        count = infile.split('.')[-2]
        outfile = out + "." + count
        command_real = command.replace('INPUT', infile).replace('OUTPUT', outfile)
        #print command_real
        output.write(command_real + "\n")

    output.close()

    return jobs_file

def submit(jobs_file, head_node, job_name, queue='all.q', mem=None):
    jobs_dir = os.path.dirname(jobs_file)
        
    if os.path.exists(jobs_file) and os.path.exists(SUBMIT_SCRIPT) and os.path.exists(jobs_dir):
        print jobs_file
        command = SUBMIT_SCRIPT + " " + head_node + " mqsub " + jobs_dir + " " + jobs_file + " " + job_name + " " + mem + " " + queue
        print command
        os.system(command)

def group_output(outdir, indir, outfile=None, eof=None, move=False, dest=None):    
    in_nums = {}
    for ff in glob.glob(os.path.join(indir, '*')):
        m = re.search('seq\.(\d+)', ff)
        if m and m.group(1):
            in_nums[m.group(1)] = ff

    out_nums = {}
    incomplete = []

    for ff in glob.glob(os.path.join(outdir, '*')):
        m = re.search('seq\.(\d+)', ff)
        if m and m.group(1):
            if eof:
                p = subprocess.Popen(["tail", "-1", ff], stdout=subprocess.PIPE)
                last_line = p.communicate()[0]

                if re.match('^\d+$', eof):
                    if int(eof) == len(last_line.split()):
                        out_nums[m.group(1)] = ff
                    else:
                        incomplete.append(m.group(1))
                else:
                    if eof in last_line:
                        out_nums[m.group(1)] = ff
                    else:
                        incomplete.append(m.group(1))
            else:
                out_nums[m.group(1)] = ff

    missing = []
    for n in in_nums.keys():
        if not out_nums.has_key(n):
            missing.append(n)

    print "missing:", ','.join(missing)

    empties = []
    for n, ff in out_nums.iteritems():
        if os.stat(ff)[6] < 1:
            print 'empty',n, ff
            empties.append(n)

    print "empties:", ','.join(empties)

    print "inputs:", len(in_nums.keys())
    print "outputs:", len(out_nums.keys())
    if eof:
        print "incomplete:", len(incomplete)

    if outfile and not empties and not missing and len(in_nums.keys()) == len(out_nums.keys()):
        out = open(outfile, 'w')
        for ff in out_nums.values():
            for line in open(ff, 'r'):
                out.write(line)
        out.close()
        
    if dest and not missing and not empties and not incomplete and len(in_nums.keys()) == len(out_nums.keys()):
        if not os.path.isdir(dest):
            os.makedirs(dest)
        if move:
            transfer(dest, os.path.split(indir)[0], cluster_dir=dest + '/cluster', job_name='rsync-align')
            
def transfer(target, source, cluster_dir=None, job_name=None, mem='1G'):
    cmd = "time rsync -aHu --stats --exclude=*.fq %s %s" % (source, target)
    cmd += ';chgrp trans-abyss %s;chmod g+w %s' % (target, target)
    cmd += ';find %s -type d |xargs chgrp trans-abyss' % (target)
    cmd += ';find %s -type d |xargs chmod g+w' % (target)
    
    print cmd
    
    if cluster_dir:
        if not os.path.isdir(cluster_dir):
            os.makedirs(cluster_dir)
        jobs_file = "%s/transfer.sh" % (cluster_dir)
        out = open(jobs_file, 'w')
        out.write(cmd + '\n')
        out.close()
        
        print "jobs file:", jobs_file
        if os.path.exists(jobs_file):
            submit(jobs_file, 'apollo', job_name, mem=mem, queue='thosts.q')
            
    else:
        os.system(cmd)
        
        
if __name__ == '__main__':
    usage = "Usage: %prog <options>"

    parser = OptionParser(usage=usage)
    parser.add_option("-g", "--group", dest="group", help="group outputs into single file given input(-I), output(-O), output-file(-o)", action="store_true", default=False)
    parser.add_option("-I", "--indir", dest="indir", help="input directory for cluster jobs")
    parser.add_option("-O", "--outdir", dest="outdir", help="output directory for cluster jobs")
    parser.add_option("-o", "--outfile", dest="outfile", help="output concatenated file")
    parser.add_option("-e", "--eof", dest="eof", help="end of file regex e.g. 'completed' for exonerate")
    parser.add_option("-m", "--move", dest="move", action="store_true", default=False)
    parser.add_option("-d", "--dest", dest="dest", help="destination folder of move")
    
    (options, args) = parser.parse_args()

    if options.group and options.indir and options.outdir and options.outfile:
        group_output(options.outdir, options.indir, options.outfile, eof=options.eof)

    else:
        group_output(options.outdir, options.indir, eof=options.eof, move=options.move, dest=options.dest)
