"""
ta-qseq2fastq.py

Original written by Richard Corbett
Modified to work with Trans-ABySS by Ka Ming Nip
Last modified on May 1, 2012
"""
import glob
import sys
import os
import re
import gzip
from optparse import OptionParser
from cStringIO import StringIO

shift31={'@':'!','A':'"','B':'#','C':'$','D':'%','E':'&','F':'\'','G':'(','H':')','I':'*','J':'+','K':',','L':'-','M':'.','N':'/','O':'0','P':'1','Q':'2','R':'3','S':'4','T':'5','U':'6','V':'7','W':'8','X':'9','Y':':','Z':';','[':'<','\\':'=',']':'>','^':'?','_':'@','`':'A','a':'B','b':'C','c':'D','d':'E','e':'F','f':'G','g':'H','h':'I','i':'J','j':'K','k':'L','l':'M','m':'N','n':'O','o':'P','p':'Q','q':'R','r':'S','s':'T','t':'U','u':'V','v':'W','w':'X','x':'Y','y':'Z','z':'\[','\{':'\\','|':']','}':'^','~':'_'}

def asciiConvert(quality,strip,striplast):

    asciList=[0]*(len(quality)-strip-striplast)
    
    for position in range(len(quality)):
        if position < strip:
            continue
        if position > len(quality) - striplast - 1:
            continue
        """
        FASTQ 33 is standard Sanger quality encoding, which means that
        you have to take the ASCII value of each character \
        in the quality string obtained from Illumina Sequencers (ascii
        64) and subtract 31 from it to get ascii 33 fastq files.
        The character is'C' == 67 and the '%' == 37. These would
        translate to Q scores of 34 and 4 which is an expected
        range of Phred scores.
        
        The quickest way to distinguish Sanger Q-score encoding
        (ASCII-33) from Illumina (Solexa) Q-score encoding (ASCII-64) \
        is to look for numerals [0-9] in the quality string. The
        numerals have ASCII values from 48-57 so it would be non-sensical
        to subtract 64 from them. If there are numerals in your quality
        string then the Q-score encoding is Sanger.
        """
        # - convert ascii 64 to ascii 33
        asciList[position]=shift31.get(quality[position], '#')
        #asciList[position] = str(unichr((ord(quality[position])-31)))
        
    return asciList

def writeFastq(fh_in, fh_out, filter_reads, strip, striplast):
    """
    Creates a fastq file from the shadow filtered qseq files
    and prints the appropriate files along with information 
    of the number of reads that were included (or excluded)
    from the final fastq files
    
    Required:
        - filehandle to the input filtered qseq files
        - filehandle to the output fastq files
        
    Returns:
        - nothing
        
    """
    tmp_data = StringIO()
    lc = 0
    total_lines = 0
    num_chsty_fail = 0
    print "Filtering Reads: %s" %filter_reads
        
    if filter_reads == True: 
    
        #for line in fh_in.readlines():
        for line in fh_in:    
            #chastity flag filtering
                    
                line = line.split("\t")
                
                if not line[10].rstrip() == "0":
                    lc += 1
                    # - only include filtered reads (1)
                    converted_ascii = "".join(asciiConvert(line[9],strip,striplast))
                    end_pos = len(line[8]) - striplast
                    tmp_data.write("@%s_%s:%s:%s:%s:%s/%s\n%s\n%s\n%s\n" % (line[0], line[1], line[2], line[3], \
                        line[4],line[5],line[7],line[8][strip:end_pos],"+", converted_ascii))                    
                        
                    # flush to disk
                    if lc == 500000:
                        fh_out.write(tmp_data.getvalue())
                        tmp_data = StringIO()
                        total_lines = total_lines + lc
                        lc = 0
                else:
                    num_chsty_fail = num_chsty_fail + 1
                    continue    
        # - write out the remainder at the end
        total_lines = total_lines + lc
        fh_out.write(tmp_data.getvalue())
            
    if filter_reads == False:
        
        #for line in fh_in.readlines():
        for line in fh_in:    
            # - include all reads (0 & 1)

            lc += 1
            line = line.split("\t")
            converted_ascii = "".join(asciiConvert(line[9],strip,striplast))
            end_pos = len(line[8]) - striplast
            tmp_data.write("@%s_%s:%s:%s:%s:%s/%s\n%s\n%s\n%s\n" % (line[0], line[1], line[2], line[3], \
                        line[4],line[5],line[7],line[8][strip:end_pos],"+", converted_ascii))
            # flush to disk
            if lc == 500000:
                fh_out.write(tmp_data.getvalue())
                tmp_data = StringIO()
                total_lines = total_lines + lc
                lc = 0
            
        # - write out the remainder at the end
        total_lines = total_lines + lc
        fh_out.write(tmp_data.getvalue())
        
            
    fh_out.close()
    print "reads included: %s" %total_lines
    print "reads excluded: %s\n" %num_chsty_fail
    
def createFastq(fullpath, cfilter, strip, striplast, filename):
    
    """
    Preparing the file directories for the subsequent function
    Note: The fastq files will be written to the same directory 
          where the qseq files are stored
    Required:
        - full path to the where the qseq files are stored
        - indication of whether the file should exclude (Filter set to True)
          or include (Filter set to False [ i.e. DON'T FILTER ]) chastity failed reads
    """
    files = "*qseq.txt"
    if filename:
        files = filename
        
    qseqpath = os.path.join(fullpath,files)
    
    chastity_filter_name = ""
    if cfilter == True:
        # - chastity passed reads only
        chastity_filter_name = "cp" 
    if cfilter == False:
        # - include all reads
        chastity_filter_name = "ca"
    
    for i in glob.glob(qseqpath):
        
        print "\nFiltering: %s" %i
        
        basname = os.path.basename(i)
        flowcell = os.path.basename(i).split("_")[0]
        lane = os.path.basename(i).split("_")[1]
        read = os.path.basename(i).split("_")[2]
        
        fout = os.path.join(fullpath,"%s_%s_%s.fastq" %(flowcell, lane, read))

        if len(os.path.basename(i).split("_")) > 5:
            barcode = os.path.basename(i).split("_")[5]
            barcode = re.sub("\..*","",barcode)
            fout = os.path.join(fullpath,"%s_%s_%s_%s.fastq" %(flowcell, lane, read, barcode))
       
        fh_in = open(i, 'r')
        fh_out = open(fout, 'w')
        
        writeFastq(fh_in, fh_out, cfilter, strip, striplast)

def writeFastqToDisk(cfilter, strip, striplast, filename, directory, reads_per_partition, prefix, gzip_out):
    if filename:
        fh_in = open(filename, 'r')
        sys.stderr.write("Reading from file " + filename + " ...\n")
    else:
        fh_in = sys.stdin
        sys.stderr.write("Reading from stdin ...\n")

    outdir = directory
    if not directory:
        outdir = os.getcwd()
    
    reads_read = 0 # for this partition only
    num_partition = 1
    if reads_per_partition > 0:
        fout = outdir + '/' + prefix + '.' + str(num_partition).zfill(4) + '.fq'
    else:
        fout = outdir + '/' + prefix + '.fq'
    
    if gzip_out:
        fout = fout + '.gz'
        fh_out = gzip.open(fout, 'w', 6)
    else:    
        fh_out = open(fout, 'w')
    sys.stderr.write("Writing to file " + fout + " ...\n")

        
    tmp_data = StringIO()
    lc = 0
    total_lines = 0
    num_chsty_fail = 0
        
    if cfilter == True: 
    
        #for line in fh_in.readlines():
        for line in fh_in:    
            #chastity flag filtering
                    
                line = line.split("\t")
                
                if not line[10].rstrip() == "0":
                    lc += 1
                    reads_read += 1
                    
                    # - only include filtered reads (1)
                    converted_ascii = "".join(asciiConvert(line[9],strip,striplast))
                    end_pos = len(line[8]) - striplast
                    tmp_data.write("@%s_%s:%s:%s:%s:%s/%s\n%s\n%s\n%s\n" % (line[0], line[1], line[2], line[3], \
                        line[4],line[5],line[7],line[8][strip:end_pos],"+", converted_ascii))                    
                        
                    # flush to disk
                    if lc == 500000:
                        fh_out.write(tmp_data.getvalue())
                        tmp_data = StringIO()
                        total_lines = total_lines + lc
                        lc = 0
                    
                    if reads_per_partition > 0 and reads_read >= reads_per_partition:
                        fh_out.write(tmp_data.getvalue())
                        tmp_data = StringIO()
                        total_lines = total_lines + lc
                        lc = 0
                        reads_read = 0
                        num_partition += 1
                        fh_out.close()
                        fout = outdir + '/' + prefix + '.' + str(num_partition).zfill(4) + '.fq'
                        if gzip_out:
                            fout = fout + '.gz'
                            fh_out = gzip.open(fout, 'w', 6)
                        else:    
                            fh_out = open(fout, 'w')
                        sys.stderr.write("Writing to file " + fout + " ...\n")
                else:
                    num_chsty_fail = num_chsty_fail + 1
                    continue    
        # - write out the remainder at the end
        total_lines = total_lines + lc
        fh_out.write(tmp_data.getvalue())
            
    if cfilter == False:
        
        #for line in fh_in.readlines():
        for line in fh_in:    
            # - include all reads (0 & 1)

            lc += 1
            reads_read += 1
            
            line = line.split("\t")
            converted_ascii = "".join(asciiConvert(line[9],strip,striplast))
            end_pos = len(line[8]) - striplast
            tmp_data.write("@%s_%s:%s:%s:%s:%s/%s\n%s\n%s\n%s\n" % (line[0], line[1], line[2], line[3], \
                        line[4],line[5],line[7],line[8][strip:end_pos],"+", converted_ascii))
            # flush to disk
            if lc == 500000:
                fh_out.write(tmp_data.getvalue())
                tmp_data = StringIO()
                total_lines = total_lines + lc
                lc = 0
                
            if reads_per_partition > 0 and reads_read >= reads_per_partition:
                fh_out.write(tmp_data.getvalue())
                tmp_data = StringIO()
                total_lines = total_lines + lc
                lc = 0
                reads_read = 0
                num_partition += 1
                fh_out.close()
                fout = outdir + '/' + prefix + '.' + str(num_partition).zfill(4) + '.fq'
                if gzip_out:
                    fout = fout + '.gz'
                    fh_out = gzip.open(fout, 'w', 6)
                else:    
                    fh_out = open(fout, 'w')
                sys.stderr.write("Writing to file " + fout + " ...\n")
            
        # - write out the remainder at the end
        total_lines = total_lines + lc
        fh_out.write(tmp_data.getvalue())
            
    fh_out.close()
    sys.stderr.write("reads included: %s\n" %total_lines)
    sys.stderr.write("reads excluded: %s\n" %num_chsty_fail)

def writeFastqToStdout(cfilter, strip, striplast, filename):
  
    if filename:
        fh_in = open(filename, 'r')
        sys.stderr.write("Reading from " + filename + " ...\n")
    else:
        fh_in = sys.stdin
        sys.stderr.write("Reading from stdin ...\n")
            
    total_lines = 0
    num_chsty_fail = 0
    
    sys.stderr.write("Writing to stdout ...\n")
    
    if cfilter == True: 
    
        #for line in fh_in.readlines():
        for line in fh_in:    
            #chastity flag filtering
                    
                line = line.split("\t")
                
                if not line[10].rstrip() == "0":
                    total_lines += 1
                    # - only include filtered reads (1)
                    converted_ascii = "".join(asciiConvert(line[9],strip,striplast))
                    end_pos = len(line[8]) - striplast
                    sys.stdout.write("@%s_%s:%s:%s:%s:%s/%s\n%s\n%s\n%s\n" % (line[0], line[1], line[2], line[3], \
                        line[4],line[5],line[7],line[8][strip:end_pos],"+", converted_ascii))                    
                else:
                    num_chsty_fail += 1
                    continue    
            
    if cfilter == False:
        
        #for line in fh_in.readlines():
        for line in fh_in:    
            # - include all reads (0 & 1)

            total_lines += 1
            line = line.split("\t")
            converted_ascii = "".join(asciiConvert(line[9],strip,striplast))
            end_pos = len(line[8]) - striplast
            sys.stdout.write("@%s_%s:%s:%s:%s:%s/%s\n%s\n%s\n%s\n" % (line[0], line[1], line[2], line[3], \
                        line[4],line[5],line[7],line[8][strip:end_pos],"+", converted_ascii))
    
    sys.stderr.write("reads included: %s\n" %total_lines)
    sys.stderr.write("reads excluded: %s\n" %num_chsty_fail)

def main():
    
    """
    Getting information from the command line
    Required:
        ...
        -f filter on or off (optional) - Default is set to False
           meaning that all reads will be included in the fastq file
        -s strip the first N base off (optional) - Default 0
        -l strip the the last N base off (optional) - Default 0 
    """
    parser = OptionParser()
    
    parser.add_option("-f", "--filter", help="filter out failed chastity read - default includes all reads", dest="cfilter", default = False, action="store_true")
    parser.add_option("-s", "--strip", metavar="INT", type="int", help="strip off first N bases - default no strip", dest="strip", default = 0, action="store")
    parser.add_option("-l", "--striplast", metavar="INT", type="int", help="strip off last N bases - default no strip", dest="striplast", default = 0, action="store")
    parser.add_option("-i", "--inputfile", dest = "filename", metavar="FILE", help="input file")
    parser.add_option("-d", "--outdir", metavar="DIR", help="output directory", dest = "directory")
    parser.add_option("-r", "--reads", metavar="LONG", type="long", dest = "reads_per_partition", default=0, action="store", help="partition output by number of reads")
    parser.add_option("-p", "--prefix", dest = "prefix", metavar="STR", help="prefix of output file(s)")
    parser.add_option("-g", "--gzip", dest="gzip_out", action="store_true", help="use gzip to compress output file(s)")
    
    (options, args)=parser.parse_args()
    
    cfilter = options.cfilter
    strip = options.strip
    striplast = options.striplast
    filename = options.filename
    directory = options.directory    
    reads_per_partition = options.reads_per_partition
    prefix = options.prefix
    gzip_out = options.gzip_out 
    
    if filename:
        if os.path.exists(filename):
            if os.path.getsize(filename) <= 0:
                sys.stderr.write("ERROR: Empty input file, " + filename)
                sys.exit(1)
        else:
            sys.stderr.write("ERROR: Cannot find input file, " + filename)
            sys.exit(1)
    
    if directory:
        if not os.path.exists(directory):
            sys.stderr.write("ERROR: Cannot find output directory, " + directory)
            sys.exit(1)        
    
    write_to_disk = directory or reads_per_partition > 0 or prefix or gzip_out      
        
    if write_to_disk:
        # we are writing output to file(s), so we need a prefix for output files
        if not prefix:
            if filename:
                # get the prefix from input file for our output file(s)
                prefix = os.path.basename(filename)                
            else:
                sys.stderr.write("ERROR: No prefix was specified for output file(s).")
                sys.exit(1)  
    
    #if options.use_io_streams:
    #    writeFastq2(cfilter, strip, striplast)
    #else:
    #    createFastq(directory, cfilter, strip, striplast, filename)
    
    sys.stderr.write("Filter Reads: %s\n" %cfilter);
    
    if write_to_disk:
        writeFastqToDisk(cfilter, strip, striplast, filename, directory, reads_per_partition, prefix, gzip_out)
    else:
        writeFastqToStdout(cfilter, strip, striplast, filename)
    
    sys.stderr.write("DONE")
main()
