from optparse import OptionParser
import os, random

def print_message(message_type, message, quit=False):
    sys.stderr.write(message_type + ": " + message + os.linesep)
    if quit:
        sys.exit(1)

def split_fasta(contigs_file, output_dir, batch_size=5000):
    lines = open(contigs_file, 'r').readlines()
    numlines = len(lines)
    if numlines % 2 == 0:
        sequences = range(1, len(lines)/2 + 1)
        random.shuffle(sequences)
        
        output_num = 1
        count = 1
        output_file = "%s/seq.%d.fa" % (output_dir, output_num)
        if os.path.exists(output_file):
            os.remove(output_file)
        out = open(output_file, 'a')
            
        for i in range(len(sequences)):
            seq_num = sequences[i]
            
            #print count, i, seq_num, output_num
            
            header = lines[seq_num * 2 - 2]
            seq = lines[seq_num * 2 - 1]
            out.write('%s%s' % (header, seq))
            
            if count == batch_size:
                out.close()
                
                count = 0
                output_num += 1
                output_file = "%s/seq.%d.fa" % (output_dir, output_num)
                out = open(output_file, 'a')
                if os.path.exists(output_file):
                    os.remove(output_file)
                    out = open(output_file, 'a')
                
            count += 1
            
        if len(sequences) % batch_size != 0:
            out.close()
    else:
        print_message('ERROR', 'The FASTA file has odd number of lines (%s).' % numlines, quit=True)


if __name__ == '__main__':   
    usage = "Usage: %prog FILE [options]"
    parser = OptionParser(usage=usage)
    parser.add_option("-b", "--batch_size", dest="batch_size", help="batch size for splitting FASTA file [default: %default]", default=5000, type='int', metavar='INT')
    parser.add_option("-o", "--output_dir", dest="output_dir", help="output directory for splitting FASTA file", metavar='DIR')
    
    (options, args) = parser.parse_args()
    
    if not args or len(args) < 1:
        print_message('ERROR', 'No FASTA file was specified.', quit=True)
        
    fasta = args[0]
    if not os.path.isfile(fasta):
        print_message('ERROR', 'Cannot find FASTA file: %s' % fasta, quit=True)
    
    batch = options.batch_size
    if batch <= 0:
        print_message('ERROR', 'The batch size specified (%s) is not a positive integer.' % batch, quit=True)
    
    if not os.path.isdir(options.output_dir):
        os.makedirs(options.output_dir)
        
    split_fasta(fasta, options.output_dir, batch)
    
